package com.bosch.tt.de.emma.connector.service;

import android.support.annotation.NonNull;
import android.util.Log;

import com.bosch.sh.connector.thirdparty.api.ThirdPartyConnectorLibrary;
import com.bosch.sh.connector.thirdparty.api.http.HttpResponseHandler;
import com.bosch.sh.connector.thirdparty.api.http.Result;

import java.io.IOException;

import okhttp3.HttpUrl;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * Created by torokt on 03/11/2017.
 */
public class RequestService {
	private static final String TAG = RequestService.class.getSimpleName();

	public static void get(@NonNull final String url,
						   @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary,
						   @NonNull final ShcCallback shcCallback) {

		Log.i(TAG, "Sending GET request to { " + url + " }");

		final Request.Builder requestBuilder = new Request.Builder().get();

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(url), requestBuilder, new HttpResponseHandler() {
			@Override
			public void onSuccess(okhttp3.Response response) {
				try {
					Log.d(TAG, "Got response from GET request " + response.code());
					Log.d(TAG, "Message: " + response.message());

					String body = response.body().string();

					Result result = new Result(response.code(), body);

					shcCallback.onResult(result);
				} catch (IOException e) {
					Log.e(TAG, "Get request has failed", e);
					shcCallback.onResult(new Result(0, "Get request has failed. Url:" + url, e));
				}
			}

			@Override
			public void onFailure(Result result) {
				Log.e(TAG, "Error during the GET request. Result: { " + result.getMessage() + " " + result.getCode() + " }");
				shcCallback.onResult(result);
			}
		});
	}

	public static void put(@NonNull final String url,
						   @NonNull final String jsonBody,
						   @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary,
						   @NonNull final ShcCallback shcCallback) {

		Log.i(TAG, "Sending PUT request to { " + url + " }");

		final Request.Builder requestBuilder = new Request.Builder().put(RequestService.createRequestBody(jsonBody));

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(url), requestBuilder, new HttpResponseHandler() {
			@Override
			public void onSuccess(Response response) {
				try {
					Log.d(TAG, "Got response from PUT request " + response.code());
					Log.d(TAG, "Message: " + response.message());

					String body = response.body().string();

					Result result = new Result(response.code(), body);

					shcCallback.onResult(result);
				} catch (IOException e) {
					Log.e(TAG, "Put request has failed", e);
					shcCallback.onResult(new Result(0, "Put request failed. Url:" + url, e));
				}
			}

			@Override
			public void onFailure(Result result) {
				Log.e(TAG, "Error during the PUT request. Result: { " + result.getMessage() + " " + result.getCode() + " }");
				shcCallback.onResult(result);
			}
		});
	}

	public static void patch(@NonNull final String url,
							 @NonNull final String jsonBody,
							 @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary,
							 @NonNull final ShcCallback shcCallback) {

		Log.i(TAG, "Sending PATCH request to { " + url + " }");
		
		final Request.Builder requestBuilder = new Request.Builder().patch(RequestService.createRequestBody(jsonBody));

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(url), requestBuilder, new HttpResponseHandler() {
			@Override
			public void onSuccess(okhttp3.Response response) {
				try {

					Log.d(TAG, "Got response from PATCH request " + response.code());
					Log.d(TAG, "Message: " + response.message());
					String body = response.body().string();

					Result result = new Result(response.code(), body);

					shcCallback.onResult(result);
				} catch (IOException e) {
					Log.e(TAG, "Patch request has failed", e);
					shcCallback.onResult(new Result(0, "Get request failed. Url:" + url, e));
				}
			}

			@Override
			public void onFailure(Result result) {
				Log.e(TAG, "Error during the PATCH request. Result: { " + result.getMessage() + " " + result.getCode() + " }");
				shcCallback.onResult(result);
			}
		});
	}
	
	private static RequestBody createRequestBody(final String jsonBody ){
		MediaType JSON = MediaType.parse("application/json; charset=utf-8");
		RequestBody requestBody;

		if (jsonBody != null) {
			requestBody = RequestBody.create(JSON, jsonBody);
		} else {
			requestBody = RequestBody.create(null, new byte[]{});
		}

		return requestBody;
	}

	private static HttpUrl.Builder createUrlBuilder(final String url ){
		String[] urlParts = url.split("\\?");

		final HttpUrl.Builder urlBuilder;

		//	This is a hack. Currently I didn't find any other solution. Maybe we should rewrite in the Javascript part 
		if (urlParts.length == 2) {
			urlBuilder = new HttpUrl.Builder()
				.addPathSegments(urlParts[0]).encodedQuery(urlParts[1]);
		} else {
			urlBuilder = new HttpUrl.Builder()
				.addPathSegments(urlParts[0]);
		}
		return urlBuilder;
	}
}
