//
//  Copyright (c) 2015-2018 Robert Bosch Smart Home GmbH
//
//  All rights reserved.
//

#import <Foundation/Foundation.h>
#import "ShcSuccessFailureBlocks.h"
#import "ShcStatusNotifications.h"
#import "ShcThirdPartyClientInfoDataSource.h"
#import "ShcRequestPort.h"
#import "ShcImportDataHolder.h"

// TODO: SH-40090 Decide the final ShcConRepresentation library states
typedef NS_ENUM(NSUInteger, ShcConRepresentationState)
{
    ShcConRepresentationNotConfigured,      // means that the instance is not configured yet (a configure method was not executed successfully)
    ShcConRepresentationNotPaired,          // means that a configure method was already called (and has a mac and ip address set locally) but the data is NOT persisted in keychain yet)
    ShcConRepresentationPaired,             // means that a configure method was already called and the data is persisted in keychain
};

typedef NS_ENUM(NSUInteger, ShcConnectivityState)
{
    ShcConnectivityUnknown,                 // We have no information about the connectivity yet
    ShcConnectivityCompatible,
    ShcConnectivitySHCOutdated,
    ShcConnectivityClientOutdated,
};

@interface ShcConRepresentation : NSObject
@property( nonatomic, assign, readonly ) ShcConRepresentationState state;
@property( nonatomic, assign, readonly ) ShcConnectivityState connectivityState;

#pragma mark - Configure default instance

- (void)configureDefaultConRepresentationWithCompletionBlock:(nullable void (^)(NSError* _Nullable error))completionBlock;

#pragma mark - Configure methods

- (void)configureWithMACAddress:(nonnull NSString*)macAddress
                completionBlock:(nullable void (^)(NSError* _Nullable error))completionBlock;

- (void)configureWithMACAddress:(nonnull NSString*)macAddress
                      ipAddress:(nonnull NSString*)ipAddress
                completionBlock:(nullable void (^)(NSError* _Nullable error))completionBlock;

#pragma mark - Request methods

- (nullable NSString*)executeRequest:(nonnull NSURLRequest*)request
                             success:(nullable ShcSuccessBlock)successBlock
                             failure:(nullable ShcFailureBlock)failureBlock;

- (nullable NSString*)executeRequest:(nonnull NSURLRequest*)request
                                port:(nonnull ShcRequestPort*)requestPort
                             success:(nullable ShcSuccessBlock)successBlock
                             failure:(nullable ShcFailureBlock)failureBlock;

- (nullable NSString*)executeRequest:(nonnull NSURLRequest*)request
           protectWithSystemPassword:(nonnull NSString*)systemPassword
                             success:(nullable ShcSuccessBlock)successBlock
                             failure:(nullable ShcFailureBlock)failureBlock;

- (nullable NSString*)executeRequest:(nonnull NSURLRequest*)request
                                port:(nonnull ShcRequestPort*)requestPort
           protectWithSystemPassword:(nonnull NSString*)systemPassword
                             success:(nullable ShcSuccessBlock)successBlock
                             failure:(nullable ShcFailureBlock)failureBlock;

#pragma mark - Request cancel methods

- (nullable NSError*)cancelRequest:(nonnull NSString*)requestID;

- (void)cancelAllRequests;

#pragma mark - Connection

- (void)refreshConnection;

#pragma mark - Rest methods

- (void)pairWithNonce:(nonnull NSString*)nonce
            pairingId:(nonnull NSString *)pairingID
           clientInfo:(nonnull id<ShcThirdPartyClientInfoDataSource>)clientInfo
              success:(nullable ShcSuccessBlock)successBlock
              failure:(nullable ShcFailureBlock)failureBlock;

#pragma mark - Connection info methods

- (nullable NSString*)macAddress;
- (nullable NSString*)ipAddress;
- (nullable NSString*)clientId;
- (nullable NSString*)homeWifiSSID;

- (BOOL)isConnected;
- (BOOL)isConnectedLocally;
- (BOOL)isConnectedRemotely;
- (BOOL)isRemoteConnectionSupported;

#pragma mark - Cleanup methods

// Removes the certificate and the pairing data from Keychain for the current instance. If it was configured with a MAC and IP address it remains configured but in ShcConRepresentationNotPaired state
- (void)clearPairData;

// Global cleanup. Removes all certificates and all pairing related data from the Keychain.
+ (void)cleanup;

@end
