//
//  RequestService.m
//  RNShcConnector
//
//  Created by Beata Brassai on 13/04/2018.
//  Copyright © 2018 Bosch TT. All rights reserved.
//

#import "RequestService.h"
#import "UserCredential.h"

@interface RequestService ()
@property( nonatomic, strong ) ShcConRepresentation* connection;
@end

#pragma mark - Initialization method

@implementation RequestService

- (nullable instancetype)initWithConRepresentation:(nonnull ShcConRepresentation*)conRepresentation notificationHandler: (id<NotificationHandler>)notificationHandler
{
    self = [super init];
    
    if( self != nil )
    {
        self.connection = conRepresentation;
        self.notificationHandler = notificationHandler;
    }
    [self notificationSubscribe];
    return( self );
}

-(void)notificationSubscribe
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(conRepresentaionDisconnected:) name:ShcConnectionDisconnectedNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(conRepresentaionConnected:) name:ShcConnectionLocalConnectionNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(conRepresentaionConnected:) name:ShcConnectionRemoteConnectionNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(conRepresentaionConnetionLost:) name:ShcConnectionLocalConnectionLostNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(conRepresentaionConnetionLost:) name:ShcConnectionRemoteConnectionLostNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(conRepresentaionDisconnected:) name:ShcConnectionNotAvailableNotification object:nil];
}

- (void) conRepresentaionConnected: (NSNotification*)notification
{
    [self.notificationHandler handleConnectedNotification:notification.name];
}

- (void) conRepresentaionConnetionLost: (NSNotification*)notification
{
    [self.notificationHandler handleConnectLostNotification:notification.name];
}

- (void) conRepresentaionDisconnected: (NSNotification*)notification
{
    [self.notificationHandler handleDisconnectedNotification:notification.name];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)pairWithNonce:(nonnull NSString*)nonce pairingId:(nonnull NSString*)pairingId success:(nullable RestSuccessBlock)successBlock failure:(nullable RestFailureBlock)failureBlock
{
    
    [self.connection pairWithNonce:nonce pairingId:pairingId clientInfo:self.createUserCredential success:^(NSURLRequest * _Nullable request, NSData * _Nullable data) {
        successBlock( [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] );
    } failure:^(NSURLRequest * _Nullable request, NSError * _Nonnull error, id  _Nullable rawResponse) {
        failureBlock(error, rawResponse);
    }];
}

- (UserCredential*)createUserCredential
{
    UserCredential* credentials = [[UserCredential alloc] init];
    
    credentials.clientName = [NSString stringWithFormat:@"EM_%@", [[UIDevice currentDevice] model]];
    credentials.os = @"IOS";
    credentials.osVersion = [[UIDevice currentDevice] systemVersion];
    
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    credentials.appVersion = [infoDictionary objectForKey:@"CFBundleShortVersionString"];
    
    return credentials;
}

- (void)sendGetByUrl:(nonnull NSString*)url successBlock:(nullable RestSuccessBlock)successBlock failureBlock:(nullable RestFailureBlock)failureBlock;
{
    NSMutableURLRequest* request = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:url]];
    [request setValue:@"application/json" forHTTPHeaderField:@"Content-type"];
    request.HTTPMethod = @"GET";
    
    [self.connection executeRequest:request success:^(NSURLRequest * _Nullable request, NSData * _Nullable data) {
        successBlock([[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
    } failure:^(NSURLRequest * _Nullable request, NSError * _Nonnull error, id  _Nullable rawResponse) {
        failureBlock(error, rawResponse);
    }];
}

- (void)sendPutByUrl:(nonnull NSString*)url body:(nullable NSString*)body successBlock:(nullable RestSuccessBlock)successBlock failureBlock:(nullable RestFailureBlock)failureBlock;
{
    NSMutableURLRequest* request = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:url]];
    [request setValue:@"application/json" forHTTPHeaderField:@"Content-type"];
    request.HTTPMethod = @"PUT";
    
    if (body != nil)
    {
        request.HTTPBody = [body dataUsingEncoding:NSUTF8StringEncoding];
    }
    
    [self.connection executeRequest:request success:^(NSURLRequest * _Nullable request, NSData * _Nullable data) {
        successBlock([[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
    } failure:^(NSURLRequest * _Nullable request, NSError * _Nonnull error, id  _Nullable rawResponse) {
        failureBlock(error, rawResponse);
    }];
}

- (void)sendPatchByUrl:(nonnull NSString*)url body:(nullable NSString*)body successBlock:(nullable RestSuccessBlock)successBlock failureBlock:(nullable RestFailureBlock)failureBlock;
{
    NSMutableURLRequest* request = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:url]];
    [request setValue:@"application/json" forHTTPHeaderField:@"Content-type"];
    request.HTTPMethod = @"PATCH";
    
    if (body != nil)
    {
        request.HTTPBody = [body dataUsingEncoding:NSUTF8StringEncoding];
    }
    
    [self.connection executeRequest:request success:^(NSURLRequest * _Nullable request, NSData * _Nullable data) {
        successBlock([[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
    } failure:^(NSURLRequest * _Nullable request, NSError * _Nonnull error, id  _Nullable rawResponse) {
        failureBlock(error, rawResponse);
    }];
}

@end
