package com.bosch.tt.privacypolicy.rn.assembler;

import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import java.util.List;

/**
 * Interface which contains the necessary conversion methods between the JSON (DTO) and the actual model classes.
 * @param <M> The model class
 */
public interface Assembler<M> {
    // Create DTO from model which can be sent to JS side
    WritableMap createWritableDTO(final M model);
    // Create model class from data coming from the JS side
    M createModel(final ReadableMap readableMap);
    // Create DTO array from model list which can be sent to JS side
    WritableArray createWritableDTOArray(final List<M> models);
    // Create model array from data coming from the JS side
    List<M> createModelArray(final ReadableArray readableArray);
}
