package com.bosch.tt.privacypolicy.rn.assembler;

import com.bosch.tt.privacypolicy.core.Country;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import java.util.ArrayList;
import java.util.List;

/**
 * Assembler class for the Country model
 */
public class CountryAssembler implements Assembler<Country> {

    private ReactContext reactContext;

    public CountryAssembler(final ReactContext reactContext) {
        this.reactContext = reactContext;
    }

    public Country createModel(ReadableMap readableMap) {
        int flagId;
        if (readableMap.hasKey("flagRes")) {
            flagId = this.reactContext.getResources().getIdentifier(readableMap.getString("flagRes"), "drawable", this.reactContext.getPackageName());
        } else {
            flagId = this.reactContext.getResources().getIdentifier("flag_de", "drawable", this.reactContext.getPackageName());
        }
        return new Country(flagId, readableMap.getString("name"), readableMap.getString("code"), readableMap.getString("language"));
    }

    @Override
    public WritableArray createWritableDTOArray(List<Country> models) {
        final WritableArray writableArray = Arguments.createArray();
        for (Country country : models) {
            writableArray.pushMap(createWritableDTO(country));
        }
        return writableArray;
    }

    @Override
    public List<Country> createModelArray(ReadableArray readableArray) {
        final List<Country> countries = new ArrayList<>();
        for (int i = 0; i < readableArray.size(); ++i) {
            countries.add(createModel(readableArray.getMap(i)));
        }
        return countries;
    }

    public WritableMap createWritableDTO(Country country) {
        final WritableMap countryDTO = Arguments.createMap();
        countryDTO.putString("flagRes", this.reactContext.getResources().getResourceEntryName(country.getFlagRes()));
        countryDTO.putString("name", country.getName());
        countryDTO.putString("code", country.getCode());
        countryDTO.putString("language", country.getLanguage());
        return countryDTO;
    }

}
