//
//  PrivacyManager.m
//  RNPrivacyComponent
//
//  Created by István Bege on 31/10/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//
#import <Foundation/Foundation.h>

#import <TTPrivacyComponent/TTPPrivacyProviderBuilder.h>

#import "PrivacyManager.h"
#import "PrivacyPolicyOfflineProvider.h"
#import "TermsOfUseProvider.h"

static PrivacyManager *privacyManager = nil;

@interface PrivacyManager()

@property TTPPrivacyProvider* termsOfUseProvider;

@end

@implementation PrivacyManager

@synthesize selectedCountry, supportedCountries, privacyProvider;

- (void)initProviderWithLeanIX:(NSString*)leanIX productionMode:(BOOL)productionMode {
    TTPPrivacyProviderBuilder* builder = [[TTPPrivacyProviderBuilder alloc] initWithLeanIX:leanIX offlineProvider:[[PrivacyPolicyOfflineProvider alloc] init]];
    if (!productionMode) {
        [builder onlineProvider:[[TTPPreviewValueProvider alloc] initWithLeanIX:leanIX]];
    }
    privacyProvider = [builder build];
}

- (id) init {
    self = [super init];
    return self;
}

+ (instancetype)getInstance
{
    static dispatch_once_t onceToken;
    
    dispatch_once(&onceToken, ^{
        privacyManager = [[self alloc] init];
    });
    return privacyManager;
}

- (TTPPrivacyProvider*) getTermsOfUseProvider {
    if (_termsOfUseProvider == nil) {
        TermsOfUseProvider* valueProvider = [[TermsOfUseProvider alloc] init];
        TTPPrivacyProviderBuilder* builder = [[TTPPrivacyProviderBuilder alloc] initWithLeanIX:nil offlineProvider:valueProvider];
        [builder onlineProvider:valueProvider];
        _termsOfUseProvider = [builder build];
    }
    return _termsOfUseProvider;
}

@end

