import React from 'react';
import PropTypes from 'prop-types';
import { requireNativeComponent, NativeModules, findNodeHandle, ViewPropTypes, Platform } from 'react-native';

const { RNPrivacyComponent, RNCountrySelectorManager } = NativeModules;

export default class CountrySelector extends React.Component {

	static propTypes = {
		...ViewPropTypes,
		countries: PropTypes.arrayOf(
			PropTypes.shape(
				{
					flagRes: PropTypes.string.isRequired,
					name: PropTypes.string.isRequired,
					code: PropTypes.string.isRequired,
					language: PropTypes.string.isRequired
				}
			)
		),
		loadCountries: PropTypes.bool,
		onCountrySelected: PropTypes.func
	}

	_viewHandle = null;

	_onCountrySelected = event => {
		if (this.props.onCountrySelected) {
			this.props.onCountrySelected(event.nativeEvent);
		}
		RNPrivacyComponent.setSelectedCountry(event.nativeEvent, this._viewHandle);
	}

	_setRef = ref => {
		this._viewHandle = ref ? findNodeHandle(ref) : null;
	}

	_setSupportedCountries() {
		RNPrivacyComponent.setSupportedCountries(this.props.countries, this._viewHandle);
		if (Platform.OS === 'ios') {
			// The iOS implementation will only show the countries if the method below gets called.
			RNCountrySelectorManager.loadCountries(this._viewHandle);
		} else {
			if (this.props.loadCountries === true) {
				RNPrivacyComponent.loadCountries(this._viewHandle);
			}
		}
	}
 
	componentDidMount() {
		if (this.props.countries) {
			this._setSupportedCountries();
		}
	}

	componentDidUpdate(prevProps) {
		if (prevProps.countries !== this.props.countries) {
			this._setSupportedCountries();
		}
	}

	render() {
		return (
			<RNCountrySelector ref={this._setRef} style={this.props.style} onCountrySelected={this._onCountrySelected} />
		);
	}
}

const RNCountrySelector = requireNativeComponent('RNCountrySelector', CountrySelector);