import React from 'react';
import PropTypes from 'prop-types';
import { requireNativeComponent, NativeModules, findNodeHandle, ViewPropTypes, Platform } from 'react-native';

const { RNPrivacyComponent, RNTTPrivacyComponentManager } = NativeModules;

export default class TTPrivacyComponent extends React.Component {

	static propTypes = {
		...ViewPropTypes,
		selectedCountry: PropTypes.shape(
			{
				flagRes: PropTypes.string,
				name: PropTypes.string.isRequired,
				code: PropTypes.string.isRequired,
				language: PropTypes.string.isRequired
			}
		),
		css: PropTypes.string,
		onDocumentLoaded: PropTypes.func,
		onScrollEnded: PropTypes.func,
		type: PropTypes.oneOf(['PrivacyPolicy', 'TermsOfUse']),
		activityIndicatorStyle: PropTypes.oneOf([
			'UIActivityIndicatorViewStyleWhiteLarge',
			'UIActivityIndicatorViewStyleWhite',
			'UIActivityIndicatorViewStyleGray'
		])
	}

	static defaultProps = {
		type: 'PrivacyPolicy',
		activityIndicatorStyle: 'UIActivityIndicatorViewStyleGray'
	}

	_viewHandle = null;

	_setRef = ref => {
		this._viewHandle = ref ? findNodeHandle(ref) : null;
	}

	_onDocumentLoaded = event => {
		if (this.props.onDocumentLoaded) {
			this.props.onDocumentLoaded(event.nativeEvent);
		}
	}

	_onScrollEnded = _event => {
		if (this.props.onScrollEnded) {
			this.props.onScrollEnded();
		}
	}

	_loadDocument() {
		if (Platform.OS === 'android') {
			this._loadDocumentAndroid();
		} else {
			this._loadDocumentIos();
		}
	}

	_loadDocumentAndroid() {
		if (this.props.type === 'TermsOfUse') {
			RNPrivacyComponent.loadTermsOfUse(this.props.selectedCountry, this._viewHandle);
		} else {
			RNPrivacyComponent.loadPrivacyPolicy(this.props.selectedCountry, this._viewHandle);
		}
	}

	_loadDocumentIos() {
		if (this.props.type === 'TermsOfUse') {
			RNTTPrivacyComponentManager.loadTermsOfUse(this.props.selectedCountry, this._viewHandle);
		} else {
			RNTTPrivacyComponentManager.loadPrivacyPolicy(this.props.selectedCountry, this._viewHandle);
		}
	}

	componentDidUpdate(prevProps) {
		if (this.props.selectedCountry !== prevProps.selectedCountry) {
			this._loadDocument();
		}
	}

	componentDidMount() {
		if (this.props.selectedCountry) {
			this._loadDocument();
		}
	}

	render() {
		return (
			<RNTTPrivacyComponent 
				ref={this._setRef}
				style={this.props.style}
				css={this.props.css}
				onDocumentLoaded={this._onDocumentLoaded}
				onScrollEnded={this._onScrollEnded}
				activityIndicatorStyle={this.props.activityIndicatorStyle}
			/>
		);
	}
}

const RNTTPrivacyComponent = requireNativeComponent('RNTTPrivacyComponent', TTPrivacyComponent);